﻿using NPOI.HSSF.UserModel;
using NPOI.OpenXml4Net.OPC.Internal;
using NPOI.SS.UserModel;
using NPOI.XSSF.UserModel;
using System;
using System.Collections.Generic;
using System.Data;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;

namespace ThinkGeoGis.Helper
{
    public static class Singleton<T> where T : class, new()
    {
        private static T _Instance;
        private static object _lockObj = new object();

        /// <summary>
        /// 获取单例对象的实例
        /// </summary>
        public static T GetInstance()
        {
            if (_Instance != null) return _Instance;
            lock (_lockObj)
            {
                if (_Instance == null)
                {
                    var temp = Activator.CreateInstance<T>();
                    System.Threading.Interlocked.Exchange(ref _Instance, temp);
                }
            }
            return _Instance;
        }
    }
    public class ExcelHelper
    {
        private static IWorkbook _workbook;
        private static ISheet _sheet;

        public static ExcelHelper Instance
        {
            get { return Singleton<ExcelHelper>.GetInstance(); }
        }

        /// <summary>
        /// 加载Excel文件
        /// </summary>
        /// <param name="excel"></param>
        private void Load(string excel)
        {
            if (!File.Exists(excel))
            {
                return;
            }
            var extension = Path.GetExtension(excel);
            var fs = new FileStream(excel, FileMode.Open, FileAccess.Read);
            switch (extension)
            {
                case ".xlsx":
                    _workbook = new XSSFWorkbook(fs);
                    break;
                case ".xls":
                    _workbook = new HSSFWorkbook(fs);
                    break;
            }
        }

        /// <summary>
        /// 获取所有Sheet Name
        /// </summary>
        public IEnumerable<string> GetAllSheets(string excel)
        {
            this.Load(excel);
            if (_workbook == null)
            {
                throw new Exception("Failed to create IWorkbook object");
            }
            var sheets = new List<string>();
            for (var i = 0; i < _workbook.NumberOfSheets; i++)
            {
                sheets.Add(_workbook.GetSheetAt(i).SheetName);
            }
            return sheets;
        }

        public DataTable ExcelToDataTable(string excel, string sheetName)
        {
            this.Load(excel);
            if (sheetName== "") return null;
            _sheet = _workbook.GetSheet(sheetName);
            var table = new DataTable();
            var fristRow = _sheet.GetRow(0); //这里的第一行必须是列头
            var totalRow = _sheet.LastRowNum;
            var totalColumn = fristRow.LastCellNum;

            // 组装列,
            for (int i = fristRow.FirstCellNum; i < totalColumn; i++)
            {
                var cell = fristRow.GetCell(i);
                if (cell != null)
                {
                    string cellValue = cell.StringCellValue;
                    if (cellValue != null)
                    {
                        var column = new DataColumn(cellValue);
                        table.Columns.Add(column);
                    }
                }
            }

            // 获取值;
            for (var i = _sheet.FirstRowNum + 1; i <= totalRow; i++)
            {
                var row = _sheet.GetRow(i);
                if (row == null) continue; //没有数据的行默认是null
                var dataRow = table.NewRow();
                for (int j = row.FirstCellNum; j < totalColumn; j++)
                {
                    if (row.GetCell(j) != null) //同理，没有数据的单元格都默认是null
                        dataRow[j] = row.GetCell(j).ToString();
                }
                table.Rows.Add(dataRow);
            }
            return table;
        }

        public Dictionary<string, List<object>> ExcelToDictionary(string excel, string sheetName)
        {
            this.Load(excel);
            if (sheetName == "") return null;
            var source = new Dictionary<string, List<object>>();
            _sheet = _workbook.GetSheet(sheetName);
            var fristRow = _sheet.GetRow(0); //这里的第一行必须是列头
            var totalRow = _sheet.LastRowNum;
            var totalColumn = fristRow.LastCellNum;
            for (int i = fristRow.FirstCellNum; i < totalColumn; i++)
            {
                var value = new List<object>();
                var cell = fristRow.GetCell(i);
                // 获取这一列的数据
                for (var j = _sheet.FirstRowNum + 1; j < totalRow; j++)
                {
                    var row = _sheet.GetRow(j);
                    value.Add(row.GetCell(i));
                }
                source.Add(cell.StringCellValue, value);
            }
            return source;
        }
    }
}
